package net.sf.javaprinciples.resource.servlet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.BeanUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.ServletRequestDataBinder;

import net.sf.javaprinciples.resource.MemoryResource;
import net.sf.javaprinciples.resource.Resource;
import net.sf.javaprinciples.resource.ResourceException;
import net.sf.javaprinciples.resource.ResourceFactory;
import net.sf.javaprinciples.resource.ResourceIdentifierParser;
import net.sf.javaprinciples.resource.ResourceParseException;

/**
 * Collection of common helper method store HTTP methods.
 *
 * @author Luka Kruscic
 */
public abstract class AbstractHandler<T> implements RequestHandler<T>
{
    private static final int BUFFER_LENGTH = 2048; // 2k characters
    private Class<?> commandClass;
    protected ResourceFactory<T> resourceFactory;
    protected ResourceIdentifierParser identifierParser;
    protected int statusCode;

    // At present we assume that all Resource representations that we will handle
    // will be character-based and not binary, except if a commandClass attribute is configured.
    // In that case, we attempt to bind request parameters to the commandClass and use that for
    // the Resource content - this is to support file uploads primarily. If this assumption
    // changes then this method will have to be changed.
    @SuppressWarnings("unchecked")
    protected Resource<T> getResourceFromRequest(HttpServletRequest request) throws ResourceException
    {
        MemoryResource<T> resource = new MemoryResource<T>();
        T content = null;
        if (null != commandClass)
        {
            Object command = BeanUtils.instantiateClass(commandClass);
            ServletRequestDataBinder binder = new ServletRequestDataBinder(command);
            binder.bind(request);
            content = (T)command;
        }
        else
        {
            try
            {
                InputStream is = request.getInputStream();
                if (null != is)
                {
                    ByteArrayOutputStream resourceBuffer = new ByteArrayOutputStream();
                    byte[] cbuf = new byte[BUFFER_LENGTH];
                    int numRead = readBytes(is, resourceBuffer, cbuf);
                    while (numRead >= 0)
                    {
                        numRead = readBytes(is, resourceBuffer, cbuf);
                    }
                    String encoding = request.getCharacterEncoding();
                    if (StringUtils.hasText(encoding))
                    {
                        content = (T)resourceBuffer.toString(encoding);
                        resource.setEncoding(encoding);
                    }
                    else
                    {
                        content = (T)resourceBuffer.toString();
                    }
                }
            }
            catch (IOException ioe)
            {
                // This is a tad dodgy, though the argument can be made that
                // being unable to read in the request stream is sort of a parsing exception.
                throw new ResourceParseException(ioe.getMessage());
            }
        }
        resource.setContent(content);
        resource.setMimeType(getMimeTypeOnly(request));
        String timeToLive = request.getHeader("X-Ttl");
        if (StringUtils.hasText(timeToLive))
        {
            try
            {
                // X-Ttl is in seconds however the lifespan is in milliseconds
                long timeToLiveValue = Long.parseLong(timeToLive) * 1000;
                resource.setLifespan(timeToLiveValue);
            }
            catch (NumberFormatException e)
            {
                // Silently ignore
            }
        }
        return resource;
    }

    private String getMimeTypeOnly(HttpServletRequest request)
    {
        String contentType = request.getContentType();
        if (!StringUtils.hasText(contentType))
        {
            return null;
        }
        int extra = contentType.indexOf(';');
        if (extra == -1)
        {
            return contentType;
        }
        return contentType.substring(0, extra);
    }

    private int readBytes(InputStream is, ByteArrayOutputStream resourceBuffer, byte[] cbuf) throws IOException
    {
        int numRead = is.read(cbuf);
        if (numRead >= 0)
        {
            resourceBuffer.write(cbuf, 0 ,numRead);
        }
        return numRead;
    }

    public void setCommandClass(Class<?> commandClass)
    {
        this.commandClass = commandClass;
    }

    public void setResourceFactory(ResourceFactory<T> resourceFactory)
    {
        this.resourceFactory = resourceFactory;
    }

    public void setIdentifierParser(ResourceIdentifierParser identifierParser)
    {
        this.identifierParser = identifierParser;
    }

    public void setStatusCode(int statusCode)
    {
        this.statusCode = statusCode;
    }
}